local VERSION = tonumber(("$Revision: 44392 $"):match("%d+"))

local Parrot = Parrot
local Parrot_TriggerConditions = Parrot:NewModule("TriggerConditions", "AceEvent-2.0", "Parser-3.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-20 21:30:47 -0400 (Fri, 20 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- #AUTODOC_NAMESPACE Parrot_TriggerConditions

local AceEvent = AceLibrary("AceEvent-2.0")

local L = Parrot:L("Parrot_TriggerConditions")

local SEAura = AceLibrary("SpecialEvents-Aura-2.0")

local _,playerClass = UnitClass("player")

local conditions = {}
local lastStates = {}
local secondaryConditions = {}

local primaryValidate = {}
local secondaryValidate = {}

local onEnableFuncs = {}
local Parrot_Triggers
function Parrot_TriggerConditions:OnEnable()
	Parrot_Triggers = Parrot:GetModule("Triggers")
	for k, v in pairs(conditions) do
		if v.getCurrent then
			lastStates[k] = v.getCurrent()
		end
	end
	
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
end

local onDisableFuncs = {}
function Parrot_TriggerConditions:OnDisable()
	for _,v in ipairs(onDisableFuncs) do
		v()
	end
end

local function RefreshEvents()
	local self = Parrot_TriggerConditions
	self:UnregisterAllEvents()
	self:UnregisterAllParserEvents()
	if not Parrot:IsModuleActive(self) then
		return
	end
	for k, v in pairs(conditions) do
		if v.events then
			for event in pairs(v.events) do
				self:RegisterEvent(event, "EventHandler")
			end
		elseif v.parserEvent then	
			self:RegisterParserEvent(v.parserEvent, "ParserEventHandler", v)
		end
	end
end

-- #NODOC
function Parrot_TriggerConditions:EventHandler(arg1)
	local event = AceEvent.currentEvent
	for k, v in pairs(conditions) do
		if v.events then
			local arg = v.events[event]
			if arg == true or (arg1 ~= nil and arg == arg1) then
				if v.getCurrent then
					local state = v.getCurrent()
					local name = v.name
					local oldState = lastStates[name]
					lastStates[name] = state
					if state ~= nil and state ~= oldState then
						self:FirePrimaryTriggerCondition(name, state)
					end
				else
					self:FirePrimaryTriggerCondition(v.name, arg1)
				end
			end
		end
	end
end

-- #NODOC
function Parrot_TriggerConditions:ParserEventHandler(data, info)
	self:FirePrimaryTriggerCondition(data.name, info[data.parserArg])
end

--[[----------------------------------------------------------------------------------
Arguments:
	string - the name of the trigger condition to fire, in English.
	[optional] string or number - a vital argument to provide information about the trigger condition.
Notes:
	* You have to register a trigger condition with :RegisterPrimaryTriggerCondition(data) first.
	* In most cases, if you use normal events in the registration or Parser-3.0, this shouldn't need to be called.
Example:
	Parrot:FirePrimaryTriggerCondition("My trigger condition", 50)
------------------------------------------------------------------------------------]]
function Parrot_TriggerConditions:FirePrimaryTriggerCondition(name, arg)
	self = Parrot_TriggerConditions -- in case someone calls Parrot:FirePrimaryTriggerCondition
	
	if Parrot_Triggers and Parrot:IsModuleActive(Parrot_Triggers) then
		Parrot_Triggers:OnTriggerCondition(name, arg)
	end
end
Parrot.FirePrimaryTriggerCondition = Parrot_TriggerConditions.FirePrimaryTriggerCondition

--[[----------------------------------------------------------------------------------
Arguments:
	table - a data table holding the details of a primary trigger condition.
Notes:
	The data table is of the following style:
	<pre>{
		name = "Name of the condition in English",
		localName = "Name of the condition in the current locale",
		events = { -- this is optional
			NAME_OF_EVENT = value, -- where NAME_OF_EVENT is the event to check, only works when value is equal to arg1. Also, value could be true in which case it is always checked.
			-- there can be multiple events.
		},
		getCurrent = function() -- this is optional and to be used with events.
			if not SomeCondition() then
				return nil -- condition won't fire.
			else
				return value -- numeric value.
			end
		end,
		parserEvent = { -- this is optional and cannot be used in tandem with events.
			eventType = "Some eventType",
			-- see Parser-3.0 for more details.
		},
		parserArg = 'argName', -- this is optional and can be used with parserEvent. It will fill the arg when firing the trigger condition.
		param = {
			-- AceOptions argument here.
			-- do not specify get, set, name, or desc.
		}
	}</pre>
Example:
	Parrot:RegisterPrimaryTriggerCondition {
		name = "Incoming block",
		localName = L["Incoming block"],
		parserEvent = {
			eventType = "Miss",
			missType = "Block",
			recipientID = "player",
		},
	}
------------------------------------------------------------------------------------]]
function Parrot_TriggerConditions:RegisterPrimaryTriggerCondition(data)
	self = Parrot_TriggerConditions -- in case someone calls Parrot:RegisterPrimaryTriggerCondition
	AceLibrary.argCheck(self, data, 2, "table")
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	local events = data.events
	local parserEvent = data.parserEvent
	if events then
		if type(events) ~= "table" then
			error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("table", type(events)), 2)
		end
		local getCurrent = data.getCurrent
		if type(getCurrent) ~= "function" and type(getCurrent) ~= "nil" then
			error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q or nil, got %q."):format("function", type(check)), 2)
		end
	elseif parserEvent then
		if type(parserEvent) ~= "table" then
			error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("table", type(parserEvent)), 2)
		end
	end
	if conditions[name] then
		error(("Trigger condition %q already registered"):format(name), 2)
	end
	conditions[name] = data
	primaryValidate[name] = localName
	RefreshEvents()
end
Parrot.RegisterPrimaryTriggerCondition = Parrot_TriggerConditions.RegisterPrimaryTriggerCondition

--[[----------------------------------------------------------------------------------
Arguments:
	table - a data table holding the details of a secondary trigger condition.
Notes:
	The data table is of the following style:
	<pre>{
		name = "Name of the condition in English",
		localName = "Name of the condition in the current locale",
		check = function(param)
			return GetSomeValue() == param
		end,
		defaultParam = 0.5, -- the default value
		param = {
			-- AceOptions argument here.
			-- do not specify get, set, name, or desc.
		}
	}</pre>
Example:
	Parrot:RegisterSecondaryTriggerCondition {
		name = "Minimum power amount",
		localName = L["Minimum power amount"],
		defaultParam = 0.5,
		param = {
			type = 'range',
			min = 0,
			max = 10000,
			step = 1,
			bigStep = 50,
		},
		check = function(param)
			if UnitIsDeadOrGhost("player") then
				return false
			end
			return UnitMana("player")/UnitManaMax("player") >= param
		end,
	}
------------------------------------------------------------------------------------]]
function Parrot_TriggerConditions:RegisterSecondaryTriggerCondition(data)
	self = Parrot_TriggerConditions -- in case someone calls Parrot:RegisterSecondaryTriggerCondition
	AceLibrary.argCheck(self, data, 2, "table")
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	secondaryConditions[name] = data
	secondaryValidate[name] = localName
	local notLocalName = data.notLocalName
	if type(notLocalName) == "string" then
		secondaryValidate["~" .. name] = notLocalName
	end
end
Parrot.RegisterSecondaryTriggerCondition = Parrot_TriggerConditions.RegisterSecondaryTriggerCondition

-- #NODOC
function Parrot_TriggerConditions:GetPrimaryConditionValidate()
	return primaryValidate
end

-- #NODOC
function Parrot_TriggerConditions:GetSecondaryConditionValidate()
	return secondaryValidate
end

-- #NODOC
function Parrot_TriggerConditions:GetPrimaryConditionParamDetails(name)
	AceLibrary.argCheck(self, name, 2, "string")
	
	local data = conditions[name]
	if not data then
		return
	end
	return data.param, data.defaultParam
end

-- #NODOC
function Parrot_TriggerConditions:GetSecondaryConditionParamDetails(name)
	AceLibrary.argCheck(self, name, 2, "string")
	
	local data = secondaryConditions[name]
	if not data then
		return
	end
	return data.param, data.defaultParam
end

-- #NODOC
function Parrot_TriggerConditions:DoesSecondaryTriggerConditionPass(name, arg)
	AceLibrary.argCheck(self, name, 2, "string")
	local notted = false
	if name:find("^~") then
		notted = true
		name = name:sub(2)
	end
	local data = secondaryConditions[name]
	if not data then
		return false
	end
	if not arg and data.param then
		return false
	end
	local value = data.check(arg)
	if notted then
		return not value
	else
		return value
	end
end

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Enemy target health percent",
	localName = L["Enemy target health percent"],
	defaultParam = 0.5,
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("target") or not UnitCanAttack("player", "target") or UnitIsDeadOrGhost("target") then
			return nil
		else
		 	return UnitHealth("target")/UnitHealthMax("target")
		end
	end,
	events = {
		UNIT_HEALTH = "target",
		UNIT_MAXHEALTH = "target",
		UNIT_FACTION = "target",
		PLAYER_TARGET_CHANGED = true,
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Friendly target health percent",
	localName = L["Friendly target health percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("target") or not UnitIsFriend("player", "target") or UnitIsDeadOrGhost("target") then
			return nil
		else
			return UnitHealth("target")/UnitHealthMax("target")
		end
	end,
	events = {
		UNIT_HEALTH = "target",
		UNIT_MAXHEALTH = "target",
		UNIT_FACTION = "target",
		PLAYER_TARGET_CHANGED = true,
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Self health percent",
	localName = L["Self health percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if UnitIsDeadOrGhost("player") then
			return nil
		else
			return UnitHealth("player")/UnitHealthMax("player")
		end
	end,
	events = {
		UNIT_HEALTH = "player",
		UNIT_MAXHEALTH = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Self mana percent",
	localName = L["Self mana percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if UnitIsDeadOrGhost("player") or UnitPowerType("player") ~= 0 then
			return nil
		else
			return UnitMana("player")/UnitManaMax("player")
		end
	end,
	events = {
		UNIT_MANA = "player",
		UNIT_MAXMANA = "player",
		UNIT_DISPLAYPOWER = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Pet health percent",
	localName = L["Pet health percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("pet") or UnitIsDeadOrGhost("pet") then
			return nil
		end
		return UnitHealth("pet")/UnitHealthMax("pet")
	end,
	events = {
		UNIT_HEALTH = "pet",
		UNIT_MAXHEALTH = "pet",
		PLAYER_PET_CHANGED = "pet",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Incoming block",
	localName = L["Incoming block"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Incoming crit",
	localName = L["Incoming crit"],
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		isCrit = true,
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Incoming dodge",
	localName = L["Incoming dodge"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Incoming parry",
	localName = L["Incoming parry"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Outgoing block",
	localName = L["Outgoing block"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Outgoing crit",
	localName = L["Outgoing crit"],
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		isCrit = true,
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Outgoing dodge",
	localName = L["Outgoing dodge"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Outgoing parry",
	localName = L["Outgoing parry"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Outgoing cast",
	localName = L["Outgoing cast"],
	parserEvent = {
		eventType = "Cast",
		sourceID = "player",
		recipientID_not = "player",
		isBegin = false,
	},
	parserArg = "abilityName",
	param = {
		type = 'text',
		usage = L["<Skill name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_TriggerConditions:RegisterParserEvent({
		eventType_in = { "Damage", "Heal" },
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	}, function(info)
		Parrot_TriggerConditions:FirePrimaryTriggerCondition("Outgoing cast", info.abilityName)
	end)
end

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Incoming cast",
	localName = L["Incoming cast"],
	parserEvent = {
		eventType = "Cast",
		recipientID = "player",
		isBegin = false,
	},
	parserArg = "abilityName",
	param = {
		type = 'text',
		usage = L["<Skill name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_TriggerConditions:RegisterParserEvent({
		eventType_in = { "Damage", "Heal" },
		recipientID = "player",
		abilityName_not = false,
	}, function(info)
		Parrot_TriggerConditions:FirePrimaryTriggerCondition("Incoming cast", info.abilityName)
	end)
end

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Self buff gain",
	localName = L["Self buff gain"],
	events = {
		SpecialEvents_PlayerBuffGained = true
	},
	param = {
		type = 'text',
		usage = L["<Buff name>"],
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Self buff fade",
	localName = L["Self buff fade"],
	events = {
		SpecialEvents_PlayerBuffLost = true
	},
	param = {
		type = 'text',
		usage = L["<Buff name>"],
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Self debuff gain",
	localName = L["Self debuff gain"],
	events = {
		SpecialEvents_PlayerDebuffGained = true
	},
	param = {
		type = 'text',
		usage = L["<Debuff name>"],
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Self debuff fade",
	localName = L["Self debuff fade"],
	events = {
		SpecialEvents_PlayerDebuffLost = true
	},
	param = {
		type = 'text',
		usage = L["<Debuff name>"],
	},
}

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Target buff gain",
	localName = L["Target buff gain"],
	param = {
		type = 'text',
		usage = L["<Buff name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_TriggerConditions:RegisterParserEvent({
		eventType = "Aura",
		isBuff = true,
	}, function(info)
		if UnitExists("target") and info.recipientName == UnitName("target") then
			Parrot_TriggerConditions:FirePrimaryTriggerCondition("Target buff gain", info.abilityName)
		end
	end)
end

Parrot_TriggerConditions:RegisterPrimaryTriggerCondition {
	name = "Target debuff gain",
	localName = L["Target debuff gain"],
	param = {
		type = 'text',
		usage = L["<Debuff name>"],
	},
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_TriggerConditions:RegisterParserEvent({
		eventType = "Aura",
		isBuff = false,
	}, function(info)
		if UnitExists("target") and info.recipientName == UnitName("target") then
			Parrot_TriggerConditions:FirePrimaryTriggerCondition("Target debuff gain", info.abilityName)
		end
	end)
end

Parrot_TriggerConditions:RegisterSecondaryTriggerCondition {
	name = "Buff inactive",
	localName = L["Buff inactive"],
	notLocalName = L["Buff active"],
	param = {
		type = 'text',
		usage = "<Buff name>",
	},
	check = function(param)
		return not SEAura:UnitHasBuff("player", param)
	end,
}

Parrot_TriggerConditions:RegisterSecondaryTriggerCondition {
	name = "Minimum power amount",
	localName = L["Minimum power amount"],
	defaultParam = 0.5,
	param = {
		type = 'range',
		min = 0,
		max = playerClass == "WARRIOR" and 100 or playerClass == "ROGUE" and 120 or 10000,
		step = 1,
		bigStep = 50,
	},
	check = function(param)
		if UnitIsDeadOrGhost("player") then
			return false
		end
		return UnitMana("player")/UnitManaMax("player") >= param
	end,
}

local spellIDs = setmetatable({}, {__index=function(self, key)
	local _, _, offset, num = GetSpellTabInfo(GetNumSpellTabs())
	local total = offset + num
	for i = 1, total do
		if key == GetSpellName(i, "spell") then
			self[key] = i
			return i
		end
	end
	self[key] = false
	return false
end})

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_TriggerConditions:RegisterEvent("SPELLS_CHANGED", function()
		for k in pairs(spellIDs) do
			spellIDs[k] = nil
		end
	end)
	for k in pairs(spellIDs) do
		spellIDs[k] = nil
	end
end

Parrot_TriggerConditions:RegisterSecondaryTriggerCondition {
	name = "Spell ready",
	localName = L["Spell ready"],
	param = {
		type = 'text',
		usage = "<Spell name>",
	},
	check = function(param)
		local id = spellIDs[param]
		if id then
			local start, duration = GetSpellCooldown(id, "spell")
			return start <= 0 or duration <= 1.5
		end
	end,
}

Parrot_TriggerConditions:RegisterSecondaryTriggerCondition {
	name = "Warrior stance",
	localName = L["Warrior stance"],
	notLocalName = L["Not in warrior stance"],
	param = {
		type = 'text',
		validate = {
			["Battle Stance"] = L["Battle Stance"],
			["Defensive Stance"] = L["Defensive Stance"],
			["Berserker Stance"] = L["Berserker Stance"],
		}
	},
	check = function(param)
		if playerClass ~= "WARRIOR" then
			return true
		end
		local form = GetShapeshiftForm(true)
		if form == 1 then
			return param == "Battle Stance"
		elseif form == 2 then
			return param == "Defensive Stance"
		elseif form == 3 then
			return param == "Berserker Stance"
		end
		return false
	end,
}
